@@ -181,9 +181,7 @@ class PlotObject:
         filename = f'{plot_name}'
         # Change the folder directory
             
-        stem = self.reaction_model.file.stem
-        
-        print(self.jupyter)
+        stem = self.reaction_model.file.stem.lstrip('<').rstrip('>')
 
         if self.jupyter:
             chart_dir = Path.cwd().joinpath('results', f'{stem}-{self.folder_name}' , 'charts', f'{self.name}')
